const ADSENSENOTIFIER_CONTRACTID = "@mozilla.org/Adsensenotifier;1";
const ADSENSENOTIFIER_CID = Components.ID( "{9B48335A-6F20-4b0a-A297-45C2D5581A88}" );
const ADSENSENOTIFIER_IID = Components.interfaces.nsIAdsensenotifier;

function nsANService() {
    this._prefs = Components.classes[ "@mozilla.org/preferences-service;1" ].getService( Components.interfaces.nsIPrefService ).getBranch( "extensions.adsense." );
    this._timer = Components.classes[ "@mozilla.org/timer;1" ].createInstance( Components.interfaces.nsITimer );
    this._stringbundle = Components.classes[ "@mozilla.org/intl/stringbundle;1" ].getService( Components.interfaces.nsIStringBundleService );
    this.bundle = this._stringbundle.createBundle( 'chrome://adsense/locale/adsense.properties' );

    // Generic Settings
    this.real_version  = "0.9.8";
    this.display_startup_page = false;
    this.is_options_open = 0;
    this.options_open = 1;
    this.listenerID = 0;
    this.windows = new Object();
    this.tooltip = this.bundle.GetStringFromName( 'tooltip.loadingtext' );
    // 1: In major callback functions
    // 2: More Detail
    this.debuglevel = 0;

    // Adsense Specific Settings
    this.loggedin = 0;
    this.login_attempts = 0;
    this.aAFC = new Array( 0, 0, 0, 0, 0, 0 );
    this.aAFS = new Array( 0, 0, 0, 0, 0, 0 );
    this.aRefer = new Array( 0, 0, 0, 0, 0, 0 );
    this.cbTimer = new Array( 'afc', 'afs', 'refer' );
    this.cbTimer[ 'afc' ] = Components.classes[ "@mozilla.org/timer;1" ].createInstance( Components.interfaces.nsITimer );
    this.cbTimer[ 'afs' ] = Components.classes[ "@mozilla.org/timer;1" ].createInstance( Components.interfaces.nsITimer );
    this.cbTimer[ 'refer' ] = Components.classes[ "@mozilla.org/timer;1" ].createInstance( Components.interfaces.nsITimer );
    this.daterangetype = 'simple';
    this.daterange_year = '';
    this.daterange_day = '';
    this.daterange_month = '';
    this.cookies = new Object();

    var self = this;
    this.myPrefObserver = {
        register: function() {
            var observerService = self._prefs.QueryInterface( Components.interfaces.nsIPrefBranchInternal );
            observerService.addObserver( "", this, false );
        },
        unregister: function() {
            var observerService = self._prefs.QueryInterface( Components.interfaces.nsIPrefBranchInternal );
            observerService.removeObserver( "", this );
        },
        observe: function( aSubject, aTopic, aData ) {
            self.prefChanged( aSubject, aTopic, aData );
        }
    }
    this.myPrefObserver.register();
    var count = { value:0 };
    var prefNames = this._prefs.getChildList( "", count );
    for ( var propName in prefNames ) {
        this.prefChanged( "", "", prefNames[ propName ] );
    }

    this._prefs.setCharPref( "current_stats", "" );
}

nsANService.prototype.prefChanged = function( aSubject, aTopic, aData ) {
    switch( this._prefs.getPrefType( aData ) ) {
        case this._prefs.PREF_STRING:
            this[aData] = this._prefs.getCharPref( aData );
            break;
        case this._prefs.PREF_INT:
            this[aData] = this._prefs.getIntPref( aData );
            break;
        case this._prefs.PREF_BOOL:
            this[aData] = this._prefs.getBoolPref( aData );
            break;
    }

    if ( aData == "pwupdate" ) {
        this.password = this.get_password();
        if ( ( this.loggedin != -1 ) && ( this.sort_by != 10 ) ) {
            this._prefs.setCharPref( "window_com", this.getListener() + ":1" );
        }
    } else if ( aData == "sort_by" ) {
        if ( this.loggedin == 1 ) {
            if ( this._prefs.getIntPref( 'sort_by' ) == 10 ) {
               this._prefs.setCharPref( "window_com", this.getListener() + ":3" );
            } else {
                this.daterangetype = 'simple';
                this.daterange_year = '';
                this.daterange_day = '';
                this.daterange_month = '';
            }
        } 
    } else if ( aData == "current_version" ) {
        if ( this.debuglevel >= 1 ) { dump( "In current version(" + this.real_version + "/" + this[aData] + ")\n" ); }
        if ( this.real_version != this[aData] ) {
            this.display_startup_page = true;
            this._prefs.setCharPref( "current_version", this.real_version );
        }
    }
}

nsANService.prototype.get_password = function() {
    var url = "chrome://adsense/";
    var passwd = "";
    try {
    if ( "@mozilla.org/passwordmanager;1" in Components.classes ) {
        if ( this.debuglevel >= 1 ) { dump( "In Comp-Get_Password (Old)\n" ); }
        var host = {value:""};
        var user =  {value:""};
        var password = {value:""}; 

        var passwordManagerInternal = Components.classes[ "@mozilla.org/passwordmanager;1" ].createInstance( Components.interfaces.nsIPasswordManagerInternal );
        passwordManagerInternal.findPasswordEntry( url, "", "", host, user, password );

        passwd = password.value;
    } else if ("@mozilla.org/login-manager;1" in Components.classes) {
        if ( this.debuglevel >= 1 ) { dump( "In Comp-Get_Password (New)\n" ); }
        var httprealm = null;
        var myLoginManager = Components.classes[ "@mozilla.org/login-manager;1" ].getService( Components.interfaces.nsILoginManager );
        var logins = myLoginManager.findLogins( {}, url, "", "" );

        for (var i = 0; i < logins.length; i++) {
            if (logins[i].username == this.username) {
                passwd = logins[i].password;
                break;
            } else if ( logins[i].username == "" ) {
                var nsLoginInfo = new Components.Constructor( "@mozilla.org/login-manager/loginInfo;1", Components.interfaces.nsILoginInfo, "init" );
                passwd = logins[i].password;
                var newLogin = new nsLoginInfo( url, url, httprealm, this.username, passwd, "", "" )
                myLoginManager.modifyLogin( logins[i], newLogin );
                break;
            }
        }
    }
    } catch( err ) {
        if ( this.debuglevel >= 1 ) { dump( "Error Getting Login/Pass: '" + err + "'\n" ); }
    }

    return( passwd );
}

nsANService.prototype.addListener = function() {
    var listenerID = ++this.listenerID;
    this.windows[listenerID] = 1;
    return( listenerID );
}

nsANService.prototype.removeListener = function( listenerID ) {
    this.windows[listenerID] = 0;
}

nsANService.prototype.getListener = function() {
    var listenerID = 0;
    for ( var propName in this.windows ) {
        if ( this.windows[propName] == 1 ) {
            listenerID = propName;
            break;
        }
    }
    return( listenerID );
}

nsANService.prototype.httprequest = function( args ) {
    if ( this.debuglevel >= 1 ) { dump( "In HTTPRequest\n" ); }
    if ( this.debuglevel >= 2 ) { dump( "  - " + args.querystring + "?" + ( args.post_data || '' ) + "\n" ); }
    var self_args = args;
    var self = this;
    var _gXMLHttpRequest = Components.classes[ "@mozilla.org/xmlextras/xmlhttprequest;1" ].createInstance( Components.interfaces.nsIXMLHttpRequest );

    var AdsenseNotifierOnload = function( request ) {
        _gXMLHttpRequest = request.target;
        self.httprequest_callback( self_args, _gXMLHttpRequest, self );
    }

    if ( self_args.method == null ) {
        self_args.method = 'GET';
    }

    if ( self_args.callback_onload ) {
        _gXMLHttpRequest.onload = function( request ) { AdsenseNotifierOnload( request ); };
    }
	 
    _gXMLHttpRequest.open( self_args.method, self_args.querystring );
    _gXMLHttpRequest.setRequestHeader( "User-Agent", "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.1.10) Gecko/20071115 Firefox/2.0.0.10" );
    _gXMLHttpRequest.setRequestHeader( "Pragma", "no-cache" );
    _gXMLHttpRequest.setRequestHeader( "Cache-Control", "no-cache" );
    if ( self_args.post_data ) {
        _gXMLHttpRequest.setRequestHeader( "Content-Type", "application/x-www-form-urlencoded" );
        _gXMLHttpRequest.send( self_args.post_data );
    } else {
        _gXMLHttpRequest.send( null );
    }
}

nsANService.prototype.httprequest_callback = function( args, request, self ) {
    if ( this.debuglevel >= 1 ) { dump( "In HTTPRequest Callback\n" ); }
    if ( request.readyState == 4 ) {
        if ( this.debuglevel >= 2 ) { dump( "Status: " + request.status + "(" + request.responseText.length + ")\n" ); }
        args.callback_parameters.responseText = request.responseText;
        args.callback_parameters.cookies = request.getResponseHeader( 'Set-Cookie' );
        request.onreadystatechange = null;
        request = null;
        args.callback_onload( args.callback_parameters, self );
    }
}

nsANService.prototype.update_cookies = function( sCookies, cookie_jar ) {
    if ( sCookies ) {
        var aCookies = sCookies.split( "\n" );
        for ( var cookie_num in aCookies ) { //For each cookie
            var aParts = aCookies[ cookie_num ].split( /;\s?/ );
            for ( var part_num in aParts ) {
                var cookie = aParts[ part_num ].match( /^(.*?)=(.*?)$/ );
                if ( cookie ) {
                    if ( !cookie[0].match( /(path)|(expires)|(domain)|(secure)/i ) ) {
                        cookie_jar[ cookie[1] ] = aCookies[ cookie_num ];
                    } else if ( cookie[0].match( /domain/i ) ) {
                    }
//dump( "Add: **" + cookie[1] + "**\n" );
                }
            }
        }
    }
}

nsANService.prototype.logon = function( args, self ) {
    if ( !self ) {
        self = this;
    }
    if ( self.debuglevel >= 1 ) { dump( "In logon (Phase: '" + args.phase + "' / Loggedin: '" + self.loggedin + "')\n" ); }

    self.update_cookies( args.cookies, self.cookies );
//dump( "Cookies: **" + self.cookies + "**\n" );
    // Display Startup page if new version.
    if ( self.display_startup_page == true ) {
        self.display_startup_page = false;
        self._prefs.setCharPref( "window_com", self.getListener() + ":5" );
    }

    if (  self.loggedin == 1 ) {
        return( self.loggedin );
    } else {
        if ( ( self.username == '' ) || ( self.password == '' ) ) {
            if ( self.debuglevel >= 1 ) { dump( "User: '" + self.username + "' / Pass: '" + "'\n" ); }
            //missing username or password, popup options.
            self._prefs.setCharPref( "window_com", self.getListener() + ":0" );
        } else if ( self.login_attempts > 3 ) {
            self.tooltip = self.bundle.GetStringFromName( 'tooltip.loginfailed' );
            self.loggedin = 0;
            self._prefs.setCharPref( "window_com", self.getListener() + ":4" );
        } else if ( ( ( self.loggedin == -1 ) && ( args.phase != 0 ) ) || ( ( self.loggedin == 0 ) && ( args.phase == 0 ) ) ) {
            self.loggedin = -1;
            args.phase++;
            self.tooltip = self.bundle.GetStringFromName( 'tooltip.loadingtext' );
            switch ( args.phase ) {
                case 1:
                    self.login_attempts++;
                    self.httprequest( { "querystring" : "https://www.google.com/accounts/ServiceLoginAuth?service=adsense&hl=en-US&ltmpl=login&ifr=true&passive=true&rm=hide&nui=3&alwf=true&continue=https%3A%2F%2Fwww.google.com%2Fadsense%2Fgaiaauth&followup=https%3A%2F%2Fwww.google.com%2Fadsense%2Fgaiaauth", "callback_onload" : self.logon, "callback_parameters" : args } );
                    break;
                case 2:
                    var re = /<input type="hidden" name="(.*?)" value="(.*?)">/ig;
                    var match = re.exec( args.responseText );
                    var post_data = '';

                    while ( match ) {
                        post_data = post_data + match[1] + '=' + self.URLencode( match[2] ) + '&';
                        match = re.exec( args.responseText );
                    }
                    var querystring = "https://www.google.com/accounts/ServiceLoginAuth";
                    post_data = post_data + 'Email=' + self.URLencode( self.username ) + '&Passwd=' + self.URLencode( self.password ) + '&null=Sign+in';
                    self.httprequest( { "querystring" : querystring, "method" : "POST", "callback_onload" : self.logon, "callback_parameters" : args, "post_data" : post_data } );
                    break;
                case 3:
                    var re = /var url = ('|")(.*?)\1;/i;
                    var match = re.exec( args.responseText );
                    if ( match ) {
                        var qs = self.hex_unencode( match[2] );
                        self.httprequest( { "querystring" : qs, "callback_onload" : self.logon, "callback_parameters" : args } );
                    } else {
                        self.tooltip = self.bundle.GetStringFromName( 'tooltip.loginfailed' );
                        self.loggedin = 0;
                        self._prefs.setCharPref( "window_com", self.getListener() + ":4" );
                    }
                    break;
                case 4:
                    self.loggedin = 1;
                    self.login_attempts = 0;
                    self.check_logon( args.responseText );
                    if ( self.sort_by == 10 ) {
                        self.set_sincelastupdate();
                    } else {
                        self.getData();
                    }
                    break;
            }
        }
    }
}

nsANService.prototype.check_logon = function( responseText ) {
    if ( this.debuglevel >= 1 ) { dump( "In check_logon\n" ); }
    var re = /(name='awglogin')|(id="acceptContractForm")/i;
    var match = re.exec( responseText );

    if ( ( match ) || ( responseText == null ) ) {
        if ( match[2] == 'id="acceptContractForm"' ) {
            this._prefs.setCharPref( "window_com", this.getListener() + ":2" );
            if ( this._timer) {
                this._timer.cancel();
            }
            return( 2 );
        } else {
            if ( this.loggedin == 1 ) {
                this.loggedin = 0;
                return( 1 );
            } else {
                return( 2 );
            }
        }
    } else {
        return( 0 );
    }
}
       
nsANService.prototype.try_logon = function( sURL ) {
    if ( !this.loggedin ) {
        var args = { "phase" : 0 };
        this.logon( args );
    }
}

nsANService.prototype.getData = function() {
    if ( this.debuglevel >= 1 ) { dump( "In getData\n" ); }
    if ( !this.loggedin ) {
        var args = { "phase" : 0 };
        this.logon( args );
    } else {
        this.setTimer();

        if ( this.product_content ) {
            this.httprequest( { "querystring" : "https://www.google.com/adsense/report/aggregate", "method" : "POST", "callback_onload" : this.getReports, "callback_parameters" : { "type" : "afc" }, "post_data" : this.create_url( { 'product' : 'afc' } ) } );
        } else {
            this.aAFC = null;
            this.aAFC = new Array(0, 0, 0, 0, 0, 0);
        }

        if ( this.product_search ) {
            this.httprequest( { "querystring" : "https://www.google.com/adsense/report/aggregate", "method" : "POST", "callback_onload" : this.getReports, "callback_parameters" : { "type" : "afs" }, "post_data" : this.create_url( { 'product' : 'afs' } ) } );
        } else {
            this.aAFS = null;
            this.aAFS = new Array(0, 0, 0, 0, 0, 0);
        }

        if ( this.product_refer ) {
            this.httprequest( { "querystring" : "https://www.google.com/adsense/report/referrals", "method" : "POST", "callback_onload" : this.update_earnings, "callback_parameters" : { "type" : "refer" }, "post_data" : this.create_url( { 'product' : 'ref' } ) } );
        } else {
            this.aRefer = null;
            this.aRefer = new Array(0, 0, 0, 0, 0, 0);
        }
    }
}

nsANService.prototype.get_desturl = function() {
    var url;

    switch ( this.openurl ) {
        case 0:
            url = 'https://www.google.com/adsense/report/overview';
            break;
        case 1:
            url = 'https://www.google.com/adsense/report/aggregate?' + this.create_url( { 'product' : 'afc' } );
            break;
        case 2:
            url = 'https://www.google.com/adsense/report/aggregate?' + this.create_url( { 'product' : 'afs' } );
            break;
        case 3:
            url = 'https://www.google.com/adsense/report/referrals?' + this.create_url( { 'product' : 'ref' } );
            break;
    }

    return( url );
}

nsANService.prototype.create_url = function( args ) {
    var simpledate = '';
    var customdate = '';

    if ( this.daterangetype == 'custom' ) {
        var current_date = new Date();
        customdate = '&dateRange.customDate.start.month=' + this.daterange_month + '&dateRange.customDate.start.day=' + this.daterange_day + '&dateRange.customDate.start.year=' + this.daterange_year + '&dateRange.customDate.end.month=' + ( current_date.getMonth() + 1 ) + '&dateRange.customDate.end.day=' + current_date.getDate() + '&dateRange.customDate.end.year=' + current_date.getFullYear();
    } else {
        simpledate = '&dateRange.simpleDate=' + this.get_sort_by( this.sort_by );
    }

    return( 'product=' + args.product + simpledate + '&dateRange.dateRangeType=' + this.daterangetype + customdate + '&unitPref=' + this.get_group_by( this.group_by ) + '&reportType=property&searchField=&groupByPref=date' );
}

nsANService.prototype.set_sincelastupdate = function( args, self ) {
    if ( !self ) {
        self = this;
    }

    if ( self._prefs.getIntPref( 'sort_by' ) == '10' ) {
        if ( !args ) {
            var oDate = new Date();
            self.httprequest( { "querystring" : 'https://www.google.com/adsense/reports-payment?begin.month=1&begin.year=2003&end.month=' + ( oDate.getMonth() + 1 ) + '&end.year=' + oDate.getFullYear() + '&null=Go', "callback_onload" : self.set_sincelastupdate, "callback_parameters" : { 'phase' : 1 } } );
        } else {
            var re = /<tr style="" class="datarow">(.*?)<\/tr>/gi;
            var match = re.exec( args.responseText );
            var last_earnings = '';
            var last_payment = '';
            var tmp_earnings = '';
            while( match ) {
//                var date = match[1].match( /<td style="text-align:left" nowrap class="" valign="top">\s*(.*?)\s*<\/td>/i );
                var date = match[1].match( /<td nowrap valign="top" style="text-align:right" class="">\s*(.*?)\s*<\/td>/i );
                if ( match[1].match( /reports-paymentdetails/i ) ) {
                    last_earnings = tmp_earnings;
                    last_payment = date[1];
                } else if ( match[1].match( /reports-earnings/i ) ) {
                    tmp_earnings = date[1];
                }
                match = re.exec( args.responseText );
            }

            if ( ( last_payment == '' ) || ( last_earnings == '' ) ) {
                self.sort_by = 3; // **Default to "All Time" since there has never been a payment
            } else {
                self.daterangetype = 'custom';
                var dates = self.standard_date( last_earnings );
                if ( dates[2] == 12 ) {
                    dates[2] = 1;
                    dates[0]++;
                } else {
                    dates[2]++;
                }
                dates[1] = 1;

                self.daterange_year = dates[0];
                self.daterange_day = dates[1];
                self.daterange_month = dates[2];
            }
            self.getData();
        }
    }
}

nsANService.prototype.tooltip_text = function() {
    if ( this.loggedin == 1 ) {
        var myTime = new Date();
        var time_diff = myTime.getTime() - this.lastupdate;
        var minutes_left = ( ( ( this.updateinterval * 60000 ) - ( time_diff + 30000 ) ) / 60000 );
        var seconds_left = ( ( ( this.updateinterval * 60000 ) - ( time_diff ) ) % 60000 ) / 1000;
        seconds_left = seconds_left.toFixed( 0 );
        if ( minutes_left <= 0 ) {
            minutes_left = 0;
        } else {
            minutes_left.toFixed( 0 );
        }
        if ( ( minutes_left <= 0 ) && ( seconds_left <= 0 ) ) {
            return( "Now Updating" );
        } else {
            var div_symbol = ':';
            if ( seconds_left < 10 ) {
                div_symbol = ':0';
            }
            return( this.bundle.GetStringFromName( 'tooltip.nextupdate' ) + ' ' + minutes_left.toFixed( 0 ) + div_symbol + seconds_left );
        }
    } else {
        return( this.tooltip );
    }
}

nsANService.prototype.update_earnings = function( args, self ) {
    if ( !self ) {
        self = this;
    }
    if ( self.debuglevel >= 1 ) { dump( "In update_earnings\n" ); }

    var check_logon = self.check_logon( args.responseText );
    if ( check_logon == 1 ) {
        var args = { "phase" : 0 };
        self.logon( args );
        return;
    } else if ( check_logon == 2 ) {
        return;
    }

    var aMatches = self.findEarnings( args.responseText );

    if ( args.type == 'afc' ) {
        self.aAFC = aMatches.slice( 0, 5 );
    }

    if ( args.type == 'afs' ) {
        self.aAFS = aMatches.slice( 0, 5 );
    }

    if ( args.type == 'refer' ) {
        self.aRefer[0] = aMatches[0];
        self.aRefer[1] = aMatches[1];
        self.aRefer[2] = aMatches[2];
        self.aRefer[3] = 0;
        self.aRefer[4] = aMatches[5];
    }

    self.update_display();
}

nsANService.prototype.getReports = function( args, self ) {
    if ( !self ) {
        self = this;
    }

    var NUM_SHORT_TRIES = 15;

    if ( args.tries ) {
        args.tries++;
    } else {
        args.tries = 1;
    }

    if ( self.debuglevel >= 1 ) { dump( "In getReports\n" ); }

    if ( ( ( args.responseText.search( "failedmessage" ) != -1 ) && args.tries < NUM_SHORT_TRIES ) || ( args.tries == 1 ) || ( ( args.tries < NUM_SHORT_TRIES ) && ( args.responseText.length == 0 ) ) ) {
        var request_re = new RegExp();
        request_re.compile( 'var reportTable = new AsyncReportTable\\(\\n\\s+(\\d+),\\n\\s+"(.*)",\\n', 'gim' );
        var request_match = request_re.exec( args.responseText );

        if ( request_match ) {
            args.report_id = request_match[1];
            args.date = request_match[2];
            args.date = args.date.replace( /\s/g, "%20" );
            args.date = args.date.replace( /,/g, "%2C" );
            args.rnd = (new Date).getTime();
        }

        var GetReportCallback = {
            notify: function cbTimerCallback( aTimer ) {
                var url = "https://www.google.com/adsense/report/online-stored-reporttable"
                var post_data = "?storedReportId=" + args.report_id + "&reportUri=%2Fadsense%2Freport%2Faggregate&formId=reportsettings&title=" + args.date + "&rnd=" + args.rnd ;
                self.httprequest( { "querystring" : url + post_data, "callback_onload" : self.getReports, "callback_parameters" : { "type" : args.type, "tries" : args.tries, "report_id" : args.report_id, "date" : args.date, "rnd" : args.rnd } } );
            }
        }

        var SHORT_SLEEP_TIME = 1500; // 1.5 seconds

        if ( self.cbTimer[ args.type ] ) {
            self.cbTimer[ args.type ].cancel();
        }

        self.cbTimer[ args.type ].initWithCallback( GetReportCallback, SHORT_SLEEP_TIME, self.cbTimer[ args.type ].TYPE_ONE_SHOT );
    } else {
        if ( args.responseText.search( args.report_id ) != -1 ) {
            args.tries = 0;
            self.update_earnings( args, self );
        }
    }
}

nsANService.prototype.update_display = function() {
    var aOutput = new Array( 0, 0, 0, 0, 0 );

    aOutput = this.aAFC.slice( 0, 5 );
    aOutput[0] = ( this.aAFC[0] || 0 ) + ( this.aAFS[0] || 0 ) + ( this.aRefer[0] || 0 );
    aOutput[1] = ( this.aAFC[1] || 0 ) + ( this.aAFS[1] || 0 ) + ( this.aRefer[1] || 0 );
    aOutput[4] = ( this.aAFC[4] || 0 ) + ( this.aAFS[4] || 0 ) + ( this.aRefer[4] || 0 );

    if ( aOutput[0] == 0 ) {
        aOutput[2] = 0;
        aOutput[3] = 0;
    } else {
        aOutput[2] = ( aOutput[1] / aOutput[0] ) * 100;
        aOutput[3] = ( aOutput[4] / aOutput[0] ) * 1000;
    }

    if ( aOutput[1] == 0 ) {
        aOutput.push( 0 );
    } else {
        aOutput.push( aOutput[4] / aOutput[1] );
    }

    var show_prefs = new Array();
    var aDisplay = new Array();
    var seperators = this.get_seperators();

    show_prefs.push( this.show_impressions );
    show_prefs.push( this.show_clicks );
    show_prefs.push( this.show_ctr );
    show_prefs.push( this.show_cpm );
    show_prefs.push( this.show_earnings );
    show_prefs.push( this.show_rpc );

    for ( i = 0; i < show_prefs.length; ++i ) {
        aOutput[i] = aOutput[i] ? aOutput[i] : 0;
        if ( show_prefs[i] ) {
            switch ( i ) {
                case 2:
                    aOutput[i] = aOutput[i].toFixed( 2 );
                    aOutput[i] = this.add_seperators( aOutput[i], '.', seperators[1], seperators[0] );
                    aOutput[i] = aOutput[i] + "%";
                    break;
                case 3:
                case 4:
                case 5:
                    aOutput[i] = aOutput[i].toFixed( 2 );
                    aOutput[i] = this.add_seperators( aOutput[i], '.', seperators[1], seperators[0] );
                    if ( seperators[2].indexOf( " " ) == 0 ) {
                        aOutput[i] = aOutput[i] + seperators[2];
                    } else {
                        aOutput[i] = seperators[2] + aOutput[i];
                    }
                    break;
                default:
                    aOutput[i] = this.add_seperators( aOutput[i], '.', seperators[1], seperators[0] );
                    break;
            }
            aDisplay.push( aOutput[i] );
        }
    }
    this._prefs.setCharPref( "current_stats", aDisplay.join( ' - ' ) );
}

nsANService.prototype.findEarnings = function( responseText ) {
    if ( this.debuglevel >= 1 ) { dump( "In findEarnings\n" ); }

    var re = new RegExp();
    re.compile( '<tr class="totals">', 'ig' );
    var lastmatch = re.exec( responseText );
    if ( lastmatch ) {
        lastmatch = lastmatch.index;
    } else {
        lastmatch = 0;
    }

    // ** Possibly remove USD in next version if still not used.
    // re.compile( '<td style="text-align:right" nowrap class="" valign="top">\\s*(?:USD|US\\$)?\\$?\\s?([(?:\\s(?!\\s))\\d\\u0660-\\u0669\\u066B\\.,]*)%?\\s?(?:\\$|US\\$)?\\s*</td>', 'ig'  );
    // 2007.11.30 10:02    re.compile( '<td nowrap valign="top" style="text-align:right" class="">\\s*(?:USD|US\\$)?\\$?\\s?([(?:\\s(?!\\s))\\d\\u0660-\\u0669\\u066B\\.,]*)%?\\s?(?:\\$|US\\$)?\\s*</td>', 'ig'  );
    // 2007.11.30 18:20    re.compile( '<td nowrap valign="top" style="text-align:right" class="">\\s*(?:USD|US\\$)?\\$?\\s?([(?:\\s(?!\\s))\\d\\u0660-\\u0669\\u066B\\.,]*)%?(?:&nbsp;)?\\s?(?:\\$|US\\$)?\\s*</td>', 'ig'  );
    re.compile( '<td (?:(?:\\s?nowrap\\s?)|(?:\\s?valign="top"\\s?)|(?:\\s?style="text-align:right"\\s?)|(?:\\s?class=""\\s?))+>\\s*(?:USD|US\\$)?\\$?\\s?([(?:\\s(?!\\s))\\d\\u0660-\\u0669\\u066B\\.,]*)%?(?:&nbsp;)?\\s?(?:(?:\\$|US\\$)|(?:\\$US))?\\s*</td>', 'ig'  );
    re.lastIndex = lastmatch;

    var aMatches = new Array( 0, 0, 0, 0, 0 );
    var cnt = 0;

    var match = re.exec( responseText );
    if ( match ) {
        while ( ( match ) && ( cnt <= 5 ) ) {
            if ( this.debuglevel >= 2 ) { dump( "Match: " + match[0] + "\n" ); }
            var temp_match;

            if ( match[1] ) {
                var arabic_re = /[\u0660-\u0669\u066B]*/;
                var arabic_match = arabic_re.exec( match[1] );
                if ( arabic_match[1] != '' ) {
                    match[1] = this.replace_arabic( match[1] );
                }
                temp_match = this.remove_commas( match[1] );
            } else {
                temp_match = 0;
            }
            aMatches[cnt] = parseFloat( temp_match ? temp_match : 0 );

            match = re.exec( responseText );
            cnt++;
        }
    }

    return( aMatches );
}

nsANService.prototype.setTimer = function() {
    var timeout = ( this.updateinterval >= 15 ) ? this.updateinterval : 15;
    timeout = timeout * 60000;

    if ( this._timer) {
        this._timer.cancel();
    }

    this._timer.initWithCallback( this, timeout, this._timer.TYPE_ONE_SHOT );
    var myTime = new Date();
    this.lastupdate = myTime.getTime();
}

nsANService.prototype.notify = function( aTimer ) {
    this.getData();
}

nsANService.prototype.URLencode = function( sStr ) {
    return escape( sStr ).replace( /\+/g, '%2B' ).replace( /[ ]/g,'+' ).replace( /\//g,'%2F' ).replace( /@/g,'%40' ).replace( /&/g, '%26' );
}

nsANService.prototype.unicode_unencode = function( sStr ) {
    var qs = sStr;
    var unicode_re = /\\(u[\da-f]{4})/ig;
    var unicode_match = unicode_re.exec( sStr );
    while ( unicode_match ) {
        var unescaped_char = unescape( '%' + unicode_match[1] );
        var match_re = new RegExp();
        match_re.compile( '\\\\' + unicode_match[1], 'i' );
        qs = qs.replace( match_re, unescaped_char );
        unicode_match = unicode_re.exec( sStr );
    }
    return( qs );
}

nsANService.prototype.hex_unencode = function( sStr ) {
    var qs = sStr;
    var unicode_re = /\\x([\da-f]{2})/ig;
    var unicode_match = unicode_re.exec( sStr );
    while ( unicode_match ) {
        var unescaped_char = unescape( '%' + unicode_match[1] );
        var match_re = new RegExp();
        match_re.compile( '\\\\x' + unicode_match[1], 'i' );
        qs = qs.replace( match_re, unescaped_char );
        unicode_match = unicode_re.exec( sStr );
    }
    return( qs );
}

nsANService.prototype.replace_arabic = function( sStr ) {
    sStr = sStr.replace( /\u0660/g, '0' );
    sStr = sStr.replace( /\u0661/g, '1' );
    sStr = sStr.replace( /\u0662/g, '2' );
    sStr = sStr.replace( /\u0663/g, '3' );
    sStr = sStr.replace( /\u0664/g, '4' );
    sStr = sStr.replace( /\u0665/g, '5' );
    sStr = sStr.replace( /\u0666/g, '6' );
    sStr = sStr.replace( /\u0667/g, '7' );
    sStr = sStr.replace( /\u0668/g, '8' );
    sStr = sStr.replace( /\u0669/g, '9' );
    sStr = sStr.replace( /\u066B/g, ',' );
    return( sStr );
}

nsANService.prototype.remove_commas = function( sStr ) {
    var re = /,\d{1,2}$/;
    if ( sStr ) {
        sStr = sStr.replace( /\s/g, '' );
        var match = re.exec( sStr );
        if ( match ) {
            sStr = sStr.replace( /,(\d{1,2})$/, '.$1' );
        }
        sStr = sStr.replace( /[\. ](\d{3})/g, '$1' );
        return sStr.replace( /,/g, '' );
    } else {
        return( "" );
    }
}

nsANService.prototype.get_seperators = function() {
    var seperator_num = 0;
    var seperators = new Array(
        Array( ',', '.', '$' ),
        Array( ',', '.', 'US$ ' ),
        Array( '.', ',', 'US$ ' ),
        Array( ' ', ',', 'US$ ' ),
        Array( ' ', ',', ' $' ),
        Array( '.', ',', 'US$ ' )
    );

    if ( this.number_locale == 0 ) {
        var locale = Components.classes[ "@mozilla.org/preferences-service;1" ].getService( Components.interfaces.nsIPrefService ).getBranch( "general.useragent." ).getCharPref( 'locale' );
        switch ( locale ) {
            case 'zh-CN': case 'zh-TW': case 'en-GB': case 'he': case 'he-IL': case 'ja': case 'ja-JP': case 'ko-KR':
                seperator_num = 1;
                break;
            case 'da': case 'da-DK': case 'nl': case 'nl-NL': case 'de': case 'de-DE': case 'it': case 'it-IT': case 'es-ES': case 'tr-TR':
                seperator_num = 2;
                break;
            case 'cs': case 'cs-CZ': case 'fi': case 'fi-FI': case 'hu': case 'hu-HU': case 'nb-NO': case 'nn-NO': case 'pl': case 'pl-PL': case 'ru': case 'ru-RU': case 'sk-SK': case 'sv-SE':
                seperator_num = 3;
                break;
            case 'fr': case 'fr-FR':
                seperator_num = 4;
                break;
            case 'pt-BR':
                seperator_num = 5;
                break;
        }
    } else {
        switch ( this.number_locale ) {
            case 1: case 6: case 11: case 14: case 15:
                seperator_num = 1;
                break;
            case 2: case 4: case 5: case 10: case 13: case 20: case 23:
                seperator_num = 2;
                break;
             case 3: case 8: case 12: case 16: case 17: case 19: case 21: case 22:
                seperator_num = 3;
                break;
             case 9:
                seperator_num = 4;
                break;
             case 18:
                seperator_num = 5;
                break;
        }
    }

    return( seperators[ seperator_num ] );
}

nsANService.prototype.add_seperators = function( nStr, inD, outD, sep ) {
    nStr += '';
    var dpos = nStr.indexOf( inD );
    var nStrEnd = '';
    if ( dpos != -1 ) {
        nStrEnd = outD + nStr.substring( dpos + 1, nStr.length );
        nStr = nStr.substring( 0, dpos );
    }
    var rgx = /(\d+)(\d{3})/;
    while ( rgx.test( nStr ) ) {
        nStr = nStr.replace( rgx, '$1' + sep + '$2' );
    }
    return nStr + nStrEnd;
}

nsANService.prototype.standard_date = function( date ) {
    var re = /(.*?)(?:-|\/|(?:\.(?! ))|(?:\. )|\s)(.*?)(?:-|\/|\.|(?:\. )|\s|(?:, ))(\d+)/i;
    var match = re.exec( date );
    var year_place, month_place, day_place;
    var dates = new Array();
    var month;

    if ( match[1].match( /\d{4}/ ) ) {
        year_place = 1;
        month_place = 2;
        day_place = 3;
    } else if ( match[1].match( /\d{2}/ ) ) {
        year_place = 3;
        month_place = 2;
        day_place = 1;
    } else {
        year_place = 3;
        month_place = 1;
        day_place = 2;
    }

    dates.push( parseFloat( match[ year_place ] ) );
    dates.push( parseFloat( match[ day_place ] ) );
    if ( match[ month_place ].match( /(?:0?1|gen|jan|janv\.|ene|Oca)/i ) ) {
        month = 1;
    } else if ( match[ month_place ].match( /(?:0?2|feb|fvr\.|Sub)/i ) ) {
        month = 2;
    } else if ( match[ month_place ].match( /(?:0?3|mar|mrt|mars)/i ) ) {
        month = 3;
    } else if ( match[ month_place ].match( /(?:0?4|apr|avr\.abr|nis)/i ) ) {
        month = 4;
    } else if ( match[ month_place ].match( /(?:0?5|mag|mei|may|mai|maj)/i ) ) {
        month = 5;
    } else if ( match[ month_place ].match( /(?:0?6|giu|jun|juin|haz)/i ) ) {
        month = 6;
    } else if ( match[ month_place ].match( /(?:0?7|lug|jul|juil\.|tem)/i ) ) {
        month = 7;
    } else if ( match[ month_place ].match( /(?:0?8|ago|aug|aot|Agu)/i ) ) {
        month = 8;
    } else if ( match[ month_place ].match( /(?:0?9|set|sep|sept\.|eyl)/i ) ) {
        month = 9;
    } else if ( match[ month_place ].match( /(?:10|ott|okt|oct|eki)/i ) ) {
        month = 10;
    } else if ( match[ month_place ].match( /(?:11|nov|nov\.|kas)/i ) ) {
        month = 11;
    } else if ( match[ month_place ].match( /(?:12|dic|dec|dc\.|des|ara)/i ) ) {
        month = 12;
    }
    dates.push( month );

    return( dates );
}

nsANService.prototype.get_sort_by = function( iSort_by ) {
    var sort_by;
    switch ( iSort_by ) {
        case 1:
            sort_by = "last7days";
            break;
        case 2:
            sort_by = "thismonth";
            break;
        case 3:
            sort_by = "alltime";
            break;
        case 4:
            sort_by = "yesterday";
            break;
        case 5:
            sort_by = "twodaysago";
            break;
        case 6:
            sort_by = "lastmonth";
            break;
        case 7:
            sort_by = "thisweek";
            break;
        case 8:
            sort_by = "lastweek";
            break;
        case 9:
            sort_by = "lastbusinessweek";
            break;
        case 10:
            sort_by = "sincelastpayment";
            break;
        default:
            sort_by = "today";
            break;
    }
    return( sort_by );
}

nsANService.prototype.get_group_by = function( iGroup_by ) {
    var group_by;
    switch ( iGroup_by ) {
        case 0:
            group_by = "page";
            break;
        case 1:
            group_by = "adunit";
            break;
    }
    return( group_by );
}

// XPCOM Stuff
function nsAdsensenotifier() {}

nsAdsensenotifier.prototype = {
    QueryInterface: function(iid) {
        if (!iid.equals(Components.interfaces.nsISupports) &&
            !iid.equals(ADSENSENOTIFIER_IID))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }
};

var nsAdsensenotifierModule = {
    registerSelf: function(compMgr, fileSpec, location, type) {
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(ADSENSENOTIFIER_CID,
                                        "nsIAdsensenotfier",
                                        ADSENSENOTIFIER_CONTRACTID,
                                        fileSpec,
                                        location,
                                        type);
    },
    getClassObject: function(compMgr, cid, iid) {
        if (!cid.equals(ADSENSENOTIFIER_CID))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        if (!iid.equals(Components.interfaces.nsIFactory))
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        return nsAdsensenotifierFactory;
    },
    canUnload: function(compMgr) { return true; }
};

var nsAdsensenotifierFactory = {
    createInstance: function(outer, iid) {
        var oPPService = new nsANService();
        if (outer != null)
            throw Components.results.NS_ERROR_NO_AGGREGATION;
        if (!iid.equals(ADSENSENOTIFIER_IID) &&
            !iid.equals(Components.interfaces.nsISupports))
            throw Components.results.NS_ERROR_INVALID_ARG;
        return oPPService;
    }
};

function NSGetModule(comMgr, fileSpec) { return nsAdsensenotifierModule; }